<?php

namespace Teknisa\MAN\Services;

class ArqPadraoRetornoService {

	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	
	public function getIntervalProd($row){
		$params = array(
				':CDARVPRODINI'=> $row['CDARVPRODINI'],
				':CDARVPRODFINAL'=>$row['CDARVPRODFINAL'],
				':CDNVPRODUTO'=> $row['CDNVPRODUTO'],
				':CDFILIAL'=> $row['CDFILIAL'],
				':NRORG'=> $row['NRORG']
			);
		return $this->entityManager->getConnection()->fetchAll(self::GET_INTERVAL_PROD, $params);
	}
	
	public function	getIntervalProdNivel($row){
		$params = array(
				':CDARVPRODINI'=> str_pad($row['PRODNIV'], 12, "0"),
				':CDARVPRODFINAL'=>  str_pad($row['PRODNIV'], 12, "9"),
				':CDNVPRODUTO'=> $row['CDNVPRODUTO'],
				':CDFILIAL'=> $row['CDFILIAL'],
				':NRORG'=> $row['NRORG']
			);
		return $this->entityManager->getConnection()->fetchAll(self::GET_INTERVAL_PROD, $params);
	}
	
	public function getIntervalProdAll($row){
		$params = array(
				':CDNVPRODUTO'=> $row['CDNVPRODUTO']
			);
		return $this->entityManager->getConnection()->fetchAll(self::GET_INTERVAL_PROD_ALL, $params);
	}
	
	
	public function getProdutosCadastrados ($CDFILIAL, $CDCAIXA, $PRODINI, $PRODFIN) {
		$params = array (
			':P_CDFILIAL' => $CDFILIAL,
			':P_CDCAIXA' => $CDCAIXA,
			':P_CDPRODINI' => $PRODINI,
			':P_CDPRODFIN' => $PRODFIN,
		);
		
		return $this->entityManager->getConnection()->fetchAll(self::GET_PROD_CADASTRADOS, $params); 
	}
	
	public function insertConsumo ($row, $CDCAIXA) {
		$params = array (
			':P_CDFILIAL' => $row['CDFILIAL'],
			':P_CDCAIXA' => $CDCAIXA,
			':P_CDPRODINI' => $row['CDARVPRODINI'],
			':P_CDPRODFIN' => $row['CDARVPRODFINAL'],
			':P_CDVERBA' => $row['CDVERBA'],
			':P_CDLOCAL' => $row['CDLOCAL'],
			'P_CDNVPRODUTO' => $row['CDNVPRODUTO']
		); 
		
		return $this->entityManager->getConnection()->executeQuery(self::INSERT_CONSUMO, $params); 
	}
	
	public function deleteConsumo ($row, $CDCAIXA) {
		$params = array (
			':P_CDFILIAL' => $row['CDFILIAL'],
			':P_CDCAIXA' => $CDCAIXA,
			':P_CDPRODINI' => $row['CDARVPRODINI'],
			':P_CDPRODFIN' => $row['CDARVPRODFINAL']
		); 
		
		return $this->entityManager->getConnection()->executeQuery(self::DELETE_CONSUMO, $params); 
	}
	
	public function buscaDadosAnalitico ($row) {
		$params = array(
	        'P_CDFILIAL' => $row['CDFILIAL'],
	        'P_CDCLIENTE' => $row['CDCLIENTE'],
	        'P_TCDCLIENTE' => $row['TCDCLIENTE'],
	        'P_CDCCUSCLIE' => $row['CDCCUSCLIE'],
	        'P_TCDCCUSCLIE' => $row['TCDCCUSCLIE'],
	        'P_DTINI' => $row['DTINI'],
	        'P_DTFIN' => $row['DTFIN'],
	        'P_CDVERBA' => $row['CDVERBA'],
	        'P_TCDVERBA' => $row['TCDVERBA'],
	        'P_CDLOCAL' => $row['CDLOCAL'],
	        'P_TCDLOCAL' => $row['TCDLOCAL'],
	        'P_CDPRODUTO' => $row['CDPRODUTO'],
	        'P_TCDPRODUTO' => $row['TCDPRODUTO'],
	        'P_CDCONSUMIDOR' => $row['CDCONSUMIDOR'],
	        'P_TCDCONSUMIDOR' => $row['TCDCONSUMIDOR']
	    );
		
		return $this->entityManager->getConnection()->fetchAll(self::ARQUIVO_ANALITICO, $params);  
	}
	
	public function buscaDadosSintetico ($row) {
		$params = array(
	        'P_CDFILIAL' => $row['CDFILIAL'],
	        'P_CDCLIENTE' => $row['CDCLIENTE'],
	        'P_TCDCLIENTE' => $row['TCDCLIENTE'],
	        'P_CDCCUSCLIE' => $row['CDCCUSCLIE'],
	        'P_TCDCCUSCLIE' => $row['TCDCCUSCLIE'],
	        'P_DTINI' => $row['DTINI'],
	        'P_DTFIN' => $row['DTFIN'],
	        'P_CDVERBA' => $row['CDVERBA'],
	        'P_TCDVERBA' => $row['TCDVERBA'],
	        'P_CDLOCAL' => $row['CDLOCAL'],
	        'P_TCDLOCAL' => $row['TCDLOCAL'],
	        'P_CDPRODUTO' => $row['CDPRODUTO'],
	        'P_TCDPRODUTO' => $row['TCDPRODUTO'],
	        'P_CDCONSUMIDOR' => $row['CDCONSUMIDOR'],
	        'P_TCDCONSUMIDOR' => $row['TCDCONSUMIDOR']
	    );
		
		return $this->entityManager->getConnection()->fetchAll(self::ARQUIVO_SINTETICO, $params);  
	}
	
	const GET_INTERVAL_PROD = "
		SELECT NRORG,
			   CDPRODUTO     AS CDPRODUTO,
			   CDARVPROD     AS CDARVPROD,
			   NMPRODUTO     AS DSBUTTON
		  FROM PRODUTO WHERE
		  CDNVPRODUTO = :CDNVPRODUTO
		  AND CDARVPROD BETWEEN :CDARVPRODINI AND :CDARVPRODFINAL 
		  AND NRORG = :NRORG"
	;
		
	const GET_INTERVAL_PROD_ALL = "
		SELECT CDPRODUTO, CDARVPROD, NMPRODUTO AS DSBUTTON
		FROM PRODUTO 
		WHERE CDNVPRODUTO = :CDNVPRODUTO";
		
	const GET_PROD_CADASTRADOS = "
		SELECT VL.CDFILIAL, VL.CDCAIXA, VL.CDPRODUTO, VL.CDVERBA, VL.CDLOCAL, PR.CDARVPROD, PR.NMPRODUTO, VE.NMVERBA, LO.NMLOCAL
		  FROM VERBALOCALPROD VL, PRODUTO PR, VERBACONSUMO VE, LOCALCONSUMO LO
		 WHERE VL.CDFILIAL  = :P_CDFILIAL
		   AND VL.CDVERBA   = VE.CDVERBA
		   AND VL.CDLOCAL   = LO.CDLOCAL
		   AND VL.CDPRODUTO = PR.CDPRODUTO
		   AND PR.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN
		   AND ((INSTR(:P_CDCAIXA, VL.CDCAIXA) > 0) OR (:P_CDCAIXA = 'T'))
		ORDER BY PR.CDARVPROD, VL.CDCAIXA
	";
	
	const INSERT_CONSUMO = "
		INSERT INTO VERBALOCALPROD (CDFILIAL, CDCAIXA, CDPRODUTO, CDVERBA, CDLOCAL)

		SELECT :P_CDFILIAL, :P_CDCAIXA, CDPRODUTO, :P_CDVERBA, :P_CDLOCAL
		  FROM PRODUTO P
		 WHERE P.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN
		   AND P.CDNVPRODUTO = :P_CDNVPRODUTO
		   AND NOT EXISTS(SELECT 1
		                    FROM VERBALOCALPROD
		                   WHERE CDFILIAL  = :P_CDFILIAL
		                     AND CDCAIXA   = :P_CDCAIXA
		                     AND CDPRODUTO = P.CDPRODUTO)
	";
	
	const DELETE_CONSUMO = "
		DELETE VERBALOCALPROD V
		WHERE V.CDFILIAL = :P_CDFILIAL
		   AND ((:P_CDCAIXA = V.CDCAIXA) OR (:P_CDCAIXA = 'T'))
		   AND EXISTS(SELECT 1
		                FROM PRODUTO
		               WHERE CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN
		                 AND CDPRODUTO = V.CDPRODUTO)
	";
	
	const ARQUIVO_ANALITICO = "
		SELECT
		   CC.CDFILIAL, CC.NMFILIAL, CC.CDCLIENTE, CC.NRINSJURCLIE, CC.NMFANTCLIE, CO.CDCONSUMIDOR, CO.CDIDCONSUMID, CO.NMCONSUMIDOR, CO.CDTIPOCONS, CC.CDCCUSCLIE,
		   CC.NMCCUSCLIE, PR.CDPRODUTO, PR.CDARVPROD, PR.NMPRODUTO, SUM(IT.QTPRODVEND) AS QTVENDIDA, IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0) AS VRUNIT,
		   SUM((TRUNC(IT.QTPRODVEND * (IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0)),2)) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS VRTOTAL,
		   VE.CDCAIXA, VE.DTVENDA, NVL(VE.NRNOTAFISCALCE, VE.NRCUPOMFIS) AS NRCUPOM, MO.CDTIPORECE, VB.NMVERBA, LO.NMLOCAL
		FROM
		   VENDA VE, ITEMVENDA IT, MOVICLIE MC, MOVCAIXA MO, PRODUTO PR, CONSUMIDOR CO,
		
		   (SELECT CC.CDCCUSCLIE, CC.NMCCUSCLIE, CF.NMFANTCLIE, CF.NRINSJURCLIE, CF.CDCLIENTE, CF.CDFILIAL, CF.NMFILIAL
		      FROM CCUSCLIE CC, (SELECT CL.NMFANTCLIE, CL.NRINSJURCLIE, CF.CDCLIENTE, CF.CDFILIAL, CF.NMFILIAL
		                           FROM CLIENTE CL, (SELECT CF.CDCLIENTE, FI.CDFILIAL, FI.NMFILIAL
		                                               FROM CLIENFILIAL CF, FILIAL FI
		                                              WHERE ((INSTR(:P_CDCLIENTE, CF.CDCLIENTE) <> 0) OR (:P_TCDCLIENTE = 'T'))
		                                                AND FI.CDFILIAL = CF.CDFILIAL
		                                                AND FI.CDFILIAL = :P_CDFILIAL) CF
		                          WHERE CL.CDCLIENTE = CF.CDCLIENTE) CF
		     WHERE CC.CDCLIENTE = CF.CDCLIENTE
		       AND ((INSTR(:P_CDCCUSCLIE, CC.CDCLIENTE||CC.CDCCUSCLIE) <> 0) OR (:P_TCDCCUSCLIE = 'T'))) CC, PARAVEND PA, VERBALOCALPROD VL, VERBACONSUMO VB, LOCALCONSUMO LO
		
		 WHERE VE.DTENTRVENDA BETWEEN :P_DTINI AND :P_DTFIN
		   AND VE.IDSITUVENDA  = 'O'
		   AND VE.CDFILIAL     = CC.CDFILIAL
		   AND IT.CDFILIAL     = VE.CDFILIAL
		   AND IT.CDCAIXA      = VE.CDCAIXA
		   AND IT.NRSEQVENDA   = VE.NRSEQVENDA
		   AND MC.CDFILIAL     = VE.CDFILIAL
		   AND MC.CDCAIXA      = VE.CDCAIXA
		   AND MC.NRSEQVENDA   = VE.NRSEQVENDA
		   AND MO.CDFILIAL     = VE.CDFILIAL
		   AND MO.CDCAIXA      = VE.CDCAIXA
		   AND MO.NRSEQVENDA   = VE.NRSEQVENDA
		   AND CC.CDCLIENTE    = CO.CDCLIENTE
		   AND CC.CDCCUSCLIE   = CO.CDCCUSCLIE
		   AND VE.CDCLIENTE    = CO.CDCLIENTE
		   AND VE.CDCONSUMIDOR = CO.CDCONSUMIDOR
		   AND PR.CDPRODUTO    = IT.CDPRODUTO
		   AND VE.CDFILIAL     = PA.CDFILIAL
		   AND VE.CDFILIAL     = VL.CDFILIAL (+)
		   AND VE.CDCAIXA      = VL.CDCAIXA  (+)
		   AND IT.CDPRODUTO    = VL.CDPRODUTO(+)
		   AND NVL(VL.CDVERBA, PA.CDVERBAPADRAO) = VB.CDVERBA(+)
		   AND NVL(VL.CDLOCAL, PA.CDLOCALPADRAO) = LO.CDLOCAL(+)
		   AND ((INSTR(:P_CDVERBA     , VB.CDVERBA                             ) <> 0) OR (:P_TCDVERBA      = 'T'))
		   AND ((INSTR(:P_CDLOCAL     , LO.CDLOCAL                             ) <> 0) OR (:P_TCDLOCAL      = 'T'))
		   AND ((INSTR(:P_CDPRODUTO   , PR.CDARVPROD                           ) <> 0) OR (:P_TCDPRODUTO    = 'T'))
		   AND ((INSTR(:P_CDCONSUMIDOR, '_'||CO.CDCLIENTE||CO.CDCONSUMIDOR||'_') <> 0) OR (:P_TCDCONSUMIDOR = 'T'))
		GROUP BY
		   CC.CDFILIAL, CC.NMFILIAL, CC.CDCLIENTE, CC.NRINSJURCLIE, CC.NMFANTCLIE, CO.CDCONSUMIDOR, CO.CDIDCONSUMID, CO.NMCONSUMIDOR, CO.CDTIPOCONS, CC.CDCCUSCLIE,
		   CC.NMCCUSCLIE, PR.CDPRODUTO, PR.CDARVPROD, PR.NMPRODUTO, IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0), VE.CDCAIXA, VE.DTVENDA, NVL(VE.NRNOTAFISCALCE, VE.NRCUPOMFIS),
		   MO.CDTIPORECE, VB.NMVERBA, LO.NMLOCAL
	";
	
	const ARQUIVO_SINTETICO = "
		SELECT
		   CC.CDFILIAL, CC.NMFILIAL, CC.CDCLIENTE, CC.NRINSJURCLIE, CC.NMFANTCLIE, CO.CDCONSUMIDOR,
		   CO.CDIDCONSUMID, CO.NMCONSUMIDOR, CO.CDTIPOCONS, CC.CDCCUSCLIE, CC.NMCCUSCLIE, SUM(IT.QTPRODVEND) AS QTVENDIDA,
		   SUM((TRUNC(IT.QTPRODVEND * (IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0)),2)) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS VRTOTAL,
		   VB.NMVERBA, LO.NMLOCAL
		FROM
		   VENDA VE, ITEMVENDA IT, MOVICLIE MO, PRODUTO PR, CONSUMIDOR CO,
		
		   (SELECT CC.CDCCUSCLIE, CC.NMCCUSCLIE, CF.NMFANTCLIE, CF.NRINSJURCLIE, CF.CDCLIENTE, CF.CDFILIAL, CF.NMFILIAL
		      FROM CCUSCLIE CC, (SELECT CL.NMFANTCLIE, CL.NRINSJURCLIE, CF.CDCLIENTE, CF.CDFILIAL, CF.NMFILIAL
		                           FROM CLIENTE CL, (SELECT CF.CDCLIENTE, FI.CDFILIAL, FI.NMFILIAL
		                                               FROM CLIENFILIAL CF, FILIAL FI
		                                              WHERE ((INSTR(:P_CDCLIENTE, CF.CDCLIENTE) <> 0) OR (:P_TCDCLIENTE = 'T'))
		                                                AND FI.CDFILIAL = CF.CDFILIAL
		                                                AND FI.CDFILIAL = :P_CDFILIAL) CF
		                          WHERE CL.CDCLIENTE = CF.CDCLIENTE) CF
		     WHERE CC.CDCLIENTE = CF.CDCLIENTE
		       AND ((INSTR(:P_CDCCUSCLIE, CC.CDCLIENTE||CC.CDCCUSCLIE) <> 0) OR (:P_TCDCCUSCLIE = 'T'))) CC, PARAVEND PA, VERBALOCALPROD VL, VERBACONSUMO VB, LOCALCONSUMO LO 
		
		 WHERE VE.DTENTRVENDA BETWEEN :P_DTINI AND :P_DTFIN
		   AND VE.IDSITUVENDA  = 'O'
		   AND VE.CDFILIAL     = CC.CDFILIAL
		   AND IT.CDFILIAL     = VE.CDFILIAL
		   AND IT.CDCAIXA      = VE.CDCAIXA
		   AND IT.NRSEQVENDA   = VE.NRSEQVENDA
		   AND MO.CDFILIAL     = VE.CDFILIAL
		   AND MO.CDCAIXA      = VE.CDCAIXA
		   AND MO.NRSEQVENDA   = VE.NRSEQVENDA
		   AND CC.CDCLIENTE    = CO.CDCLIENTE
		   AND CC.CDCCUSCLIE   = CO.CDCCUSCLIE
		   AND VE.CDCLIENTE    = CO.CDCLIENTE
		   AND VE.CDCONSUMIDOR = CO.CDCONSUMIDOR
		   AND PR.CDPRODUTO    = IT.CDPRODUTO
		   AND VE.CDFILIAL     = PA.CDFILIAL
		   AND VE.CDFILIAL     = VL.CDFILIAL (+)
		   AND VE.CDCAIXA      = VL.CDCAIXA  (+)
		   AND IT.CDPRODUTO    = VL.CDPRODUTO(+)
		   AND NVL(VL.CDVERBA, PA.CDVERBAPADRAO) = VB.CDVERBA(+)
		   AND NVL(VL.CDLOCAL, PA.CDLOCALPADRAO) = LO.CDLOCAL(+)
		   AND ((INSTR(:P_CDVERBA     , VB.CDVERBA                             ) <> 0) OR (:P_TCDVERBA      = 'T'))
		   AND ((INSTR(:P_CDLOCAL     , LO.CDLOCAL                             ) <> 0) OR (:P_TCDLOCAL      = 'T'))
		   AND ((INSTR(:P_CDPRODUTO   , PR.CDARVPROD                           ) <> 0) OR (:P_TCDPRODUTO    = 'T'))
		   AND ((INSTR(:P_CDCONSUMIDOR, '_'||CO.CDCLIENTE||CO.CDCONSUMIDOR||'_') <> 0) OR (:P_TCDCONSUMIDOR = 'T'))
		GROUP BY
		   CC.CDFILIAL, CC.NMFILIAL, CC.CDCLIENTE, CC.NRINSJURCLIE, CC.NMFANTCLIE, CO.CDCONSUMIDOR,
		   CO.CDIDCONSUMID, CO.NMCONSUMIDOR, CO.CDTIPOCONS, CC.CDCCUSCLIE, CC.NMCCUSCLIE, VB.NMVERBA, LO.NMLOCAL
	";
}