<?php

namespace Teknisa\MAN\Services;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\Util\Functions;

class ImportacaoConsumidorProcessaService {
	
	protected $entityManager;
	protected $util;
	protected $boImpMercadoBoca;
	protected $evoluiSaldoService;
	protected $criptoPasswordUrl;
    protected $xmlSchemaUrl;
    protected $novoCodigo;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager, $util, Services\EvoluiSaldo $evoluiSaldoService, $criptoPasswordUrl, $xmlSchemaUrl, Services\NovoCodigo $NovoCodigo)  {
		$this->entityManager = $entityManager;
		$this->util = $util;
		$this->evoluiSaldoService = $evoluiSaldoService;
        $this->criptoPasswordUrl     = $criptoPasswordUrl;
        $this->xmlSchemaUrl          = $xmlSchemaUrl;
        $this->novoCodigo = $NovoCodigo;
	}
	
	public function findAjustaSaldo ($CDFILIAL) {
		$sql = "SELECT IDAJUSALDIMPCON FROM PARAVEND
    			WHERE CDFILIAL = :CDFILIAL";
	    $params = [
	        ':CDFILIAL' => $CDFILIAL
	    ];
	    return $this->entityManager->getConnection()->fetchOne($sql, $params);
	}
    	
	public function verificarDuplicidade($consumers, $filial) {
        $log = [];
        $boParaImp = false;
        $boInclui_Log = false;

        // Load existing ID cards associated with the branch
        $existingCracha = $this->getCrachaFilial($filial);

        // Create indices for efficient lookup
        $existingCrachaIndex = [];
        foreach ($existingCracha as $row) {
            $existingCrachaIndex[$row['CDIDCONSUMID']] = $row;
        }

        $consumers = $this->checkDuplicateAndGenerateLog($consumers, $log, $boParaImp, $boInclui_Log, $existingCrachaIndex, $filial);
        
        // Generate log for consumers
        $this->geraLog($consumers, $log, $boParaImp, $boInclui_Log);
        
        // Ordena o array log pelo campo DSLOGIMP
        usort($log, function($a, $b) {
            return strcmp($a['DSLOGIMP'], $b['DSLOGIMP']);
        });

        return [$log, $boParaImp, $boInclui_Log, $consumers];
    }

    protected function getCrachaFilial($filial) {
        $query = "
            SELECT C.CDCLIENTE, C.CDCONSUMIDOR, C.CDIDCONSUMID, F.CDFILIAL
            FROM CONSUMIDOR C, CONSUMIFILIAL F
            WHERE F.CDFILIAL = :P_CDFILIAL
            AND C.CDCLIENTE = F.CDCLIENTE
            AND C.CDCONSUMIDOR = F.CDCONSUMIDOR
            AND C.IDSITCONSUMI = '1'
        ";
        $params = [':P_CDFILIAL' => $filial];
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }

    protected function checkDuplicateAndGenerateLog($consumers, &$log, &$boParaImp, &$boInclui_Log, $existingCrachaIndex, $filial) {
        $indexCd = [];
        $indexId = [];
        $loggedCd = [];
        $loggedId = [];
    
        foreach ($consumers as &$consumer) {
            $cdKey = $consumer['CDCLIENTE'] . $consumer['CDCONSUMIDOR'];
            $idKey = $consumer['CDIDCONSUMID'];
    
            // Verifica se tem codigo (CDCLIENTE + CDCONSUMIDOR) duplicado no arquivo
            if (isset($indexCd[$cdKey])) {
                // Log para todos os consumidores duplicados anteriores
                if (!isset($loggedCd[$cdKey])) {
                    foreach ($indexCd[$cdKey] as &$prevConsumer) {
                        $log[] = $this->generateLogEntry($prevConsumer, 'Cod_Duplicado');
                        $prevConsumer['IDVALIDO'] = 'N';
                    }
                    $loggedCd[$cdKey] = true;
                }
                // Log para o consumidor atual
                $log[] = $this->generateLogEntry($consumer, 'Cod_Duplicado');
                $boParaImp = true;
                $boInclui_Log = true;
                $consumer['IDVALIDO'] = 'N';
                $indexCd[$cdKey][] = &$consumer;
            } else {
                $indexCd[$cdKey] = [&$consumer];
            }
    
            // Verifica se tem crachá (CDIDCONSUMID) duplicado no arquivo
            if (isset($indexId[$idKey])) {
                // Log para todos os consumidores duplicados anteriores
                if (!isset($loggedId[$idKey])) {
                    foreach ($indexId[$idKey] as &$prevConsumer) {
                        $log[] = $this->generateLogEntry($prevConsumer, 'Cracha_Duplicado');
                        $prevConsumer['IDVALIDO'] = 'N';
                    }
                    $loggedId[$idKey] = true;
                }
                // Log para o consumidor atual
                $log[] = $this->generateLogEntry($consumer, 'Cracha_Duplicado');
                $boParaImp = true;
                $boInclui_Log = true;
                $consumer['IDVALIDO'] = 'N';
                $indexId[$idKey][] = &$consumer;
            } else {
                $indexId[$idKey] = [&$consumer];
            }
    
            // Verifica se o crachá (CDIDCONSUMID) já existe na filial, para outro CDCLIENTE ou CDCONSUMIDOR
            // Se for o mesmo CDCLIENTE e CDCONSUMIDOR é o mesmo consumidor, então não gera erro
            if ($this->isCrachaFilialDuplicate($consumer, $existingCrachaIndex, $filial)) {
                $log[] = $this->generateLogEntry($consumer, 'CrachaFilial');
                $boParaImp = true;
                $boInclui_Log = true;
                $consumer['IDVALIDO'] = 'N';
            }
        }
    
        return $consumers;
    }

    protected function isCrachaFilialDuplicate($consumer, $existingCrachaIndex, $filial) {
        if (isset($existingCrachaIndex[$consumer['CDIDCONSUMID']])) {
            $cracha = $existingCrachaIndex[$consumer['CDIDCONSUMID']];
            return $cracha['CDFILIAL'] === $filial &&
                   ($cracha['CDCLIENTE'] !== $consumer['CDCLIENTE'] || $cracha['CDCONSUMIDOR'] !== $consumer['CDCONSUMIDOR']);
        }
        return false;
    }

    protected function generateLogEntry($consumer, $message) {
        return [
            'DSLOGIMP' => $this->getMessage($message),
            'CDCLIENTE' => $consumer['CDCLIENTE'],
            'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR'],
            'CDIDCONSUMID' => $consumer['CDIDCONSUMID'],
            'CDEXCONSUMID' => $consumer['CDEXCONSUMID'],
            'NMCONSUMIDOR' => $consumer['NMCONSUMIDOR'],
            'CDTIPOCONS' => $consumer['CDTIPOCONS'],
            'CDCCUSCLIE' => $consumer['CDCCUSCLIE'],
            'NRCPFRESPCON' => $consumer['NRCPFRESPCON']
        ];
    }

    protected function getMessage($code) {
        // Simulating message retrieval based on code
        $messages = [
            'Cod_Duplicado' => 'Código do Consumidor em duplicidade no arquivo.',
            'Cracha_Duplicado' => 'Código de Identificação em duplicidade no arquivo.',
            'CrachaFilial' => 'Crachá já associado a esta filial.',
            'CdCliNull' => 'O Código do Cliente não pode ser vazio.',
            'CdCliNaoEncontrado' => 'Código do Cliente não encontrado.',
            'CliNaoAssociado' => 'Cliente não associado à filial de importação.',
            'CdConsNull' => 'O Código do Consumidor não pode ser vazio.',
            'NmConsNull' => 'O Nome do Consumidor não pode ser vazio.',
            'CdTipoCons' => 'O Tipo de Consumidor não pode ser vazio.',
            'CdCCusClie' => 'O Centro de Custo não pode ser vazio.',
            'CdTipoCons1' => 'Código de Tipo do Consumidor Não Encontrado. Será incluído automaticamente.',
            'CdCCusClie1' => 'Código de Centro de Custo Não Encontrado. Será incluído automaticamente.',
            'CdIdJaCad_log' => 'Identificador do consumidor já existe no cadastro / troca de crachá.',
            'CpfInvalido' => 'CPF Inválido. Será incluído automaticamente como vazio.'
        ];
        return $messages[$code] ?? 'Mensagem desconhecida';
    }

    protected function geraLog($consumers, &$log, &$boParaImp, &$boInclui_Log) {
        foreach ($consumers as $consumer) {
            if ($consumer['CDCLIENTE'] === 'VAZIO') {
                $log[] = $this->generateLogEntry($consumer, 'CdCliNull');
                $boParaImp = true;
                $boInclui_Log = true;
            } else if ($consumer['IDMOTIVO'] === 'NAO_ENC') {
                $log[] = $this->generateLogEntry($consumer, 'CdCliNaoEncontrado');
                $boParaImp = true;
                $boInclui_Log = true;
            } else  if ($consumer['IDMOTIVO'] === 'CLI_FIL') {
                $log[] = $this->generateLogEntry($consumer, 'CliNaoAssociado');
                $boParaImp = true;
                $boInclui_Log = true;
            }

            if (empty($consumer['CDCONSUMIDOR'])) {
                $log[] = $this->generateLogEntry($consumer, 'CdConsNull');
                $boParaImp = true;
                $boInclui_Log = true;
            }

            if (empty($consumer['NMCONSUMIDOR'])) {
                $log[] = $this->generateLogEntry($consumer, 'NmConsNull');
                $boParaImp = true;
                $boInclui_Log = true;
            }

            if (empty($consumer['CDTIPOCONS'])) {
                $log[] = $this->generateLogEntry($consumer, 'CdTipoCons');
                $boParaImp = true;
                $boInclui_Log = true;
            }

            if (empty($consumer['CDCCUSCLIE'])) {
                $log[] = $this->generateLogEntry($consumer, 'CdCCusClie');
                $boParaImp = true;
                $boInclui_Log = true;
            }

            if (!empty($consumer['CDTIPOCONS'])) {
                $result = $this->queryTipoCons($consumer['CDTIPOCONS']);
                if (empty($result)) {
                    $boInclui_Log = true;
                    $log[] = $this->generateLogEntry($consumer, 'CdTipoCons1');
                }
            }

            if (!empty($consumer['CDCCUSCLIE'])) {
                $result = $this->queryCentCusto($consumer['CDCLIENTE'], $consumer['CDCCUSCLIE']);
                if (empty($result)) {
                    $boInclui_Log = true;
                    $log[] = $this->generateLogEntry($consumer, 'CdCCusClie1');
                }
            }
            
            if ($consumer['IDINACONSARQIMP'] === 'N') {
                $result = $this->queryVrIdentificador($consumer['CDIDCONSUMID'], $consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
                if ($result['NRREGISTROS'] > 0) {
                    $log[] = $this->generateLogEntry($consumer, 'CdIdJaCad_log');
                    $boParaImp = true;
                    $boInclui_Log = true;
                }
            }

            if (!empty($consumer['NRCPFRESPCON']) && (!$this->isCpfValid($consumer['NRCPFRESPCON']) || strlen($consumer['NRCPFRESPCON']) !== 11)) {
                $boInclui_Log = true;
                $log[] = $this->generateLogEntry($consumer, 'CpfInvalido');
            }
        }
    }

    protected function queryTipoCons($cdTipoCons) {
	    $query = "
	        SELECT CDTIPOCONS
	        FROM TIPOCONS
	        WHERE CDTIPOCONS = :P_CDTIPOCONS
	    ";
	    $params = [':P_CDTIPOCONS' => $cdTipoCons];
	    return $this->entityManager->getConnection()->fetchAll($query, $params);
	}


    protected function queryCentCusto($cdCliente, $cdCCusClie) {
	    $query = "
	        SELECT CDCLIENTE, CDCCUSCLIE, NMCCUSCLIE
	        FROM CCUSCLIE
	        WHERE CDCLIENTE = :P_CDCLIENTE
	        AND CDCCUSCLIE = :P_CDCCUSCLIE
	    ";
	    $params = [':P_CDCLIENTE' => $cdCliente, ':P_CDCCUSCLIE' => $cdCCusClie];
	    return $this->entityManager->getConnection()->fetchAll($query, $params);
	}

    protected function queryVrIdentificador($cdIdConsumid, $cdCliente, $cdConsumidor) {
        $query = "
            SELECT COUNT(*) AS NRREGISTROS
            FROM CONSUMIDOR C
            WHERE C.CDIDCONSUMID = :P_CDIDCONSUMID
            AND C.CDIDCONSUMID IS NOT NULL
            AND C.IDSITCONSUMI = '1'
            AND C.CDCLIENTE = :P_CDCLIENTE
            AND C.CDCONSUMIDOR <> :P_CDCONSUMIDOR
        ";
        $params = [':P_CDIDCONSUMID' => $cdIdConsumid, ':P_CDCLIENTE' => $cdCliente, ':P_CDCONSUMIDOR' => $cdConsumidor];
        return $this->entityManager->getConnection()->fetchAssoc($query, $params);
    }

    protected function isCpfValid($cpf) {
    	return $this->util->validarCPF($cpf);
    }
    
    
    /*
    --------------------------------------------------------------------------------
    --------------------------- INSERÇÃO DE CONSUMIDOR -----------------------------
    --------------------------------------------------------------------------------
    */
    
    public function setConsumidor($consumers) {
        // Placeholder for the result
        $result = [];
        
        // Variáveis de controle
        $this->boImpMercadoBoca = false;
        
        $CDLAYOUTIMP = isset($consumers[0]) ? $consumers[0]['CDLAYOUTIMP'] : null;
        $NRSEQTABELAY = isset($consumers[0]) ? $consumers[0]['NRSEQTABELAY'] : null;
        $CDFILIAL = isset($consumers[0]) ? $consumers[0]['CDFILIAL'] : null;
        $FILENAME = isset($consumers[0]) ? $consumers[0]['FILENAME'] : null;
        $AJUSTASALDOCHECK = isset($consumers[0]) ? $consumers[0]['AJUSTASALDOCHECK'] : null;
        $CDOPERADOR = isset($consumers[0]) ? $consumers[0]['CDOPERADOR'] : null;
        $NRORG = isset($consumers[0]) ? $consumers[0]['NRORG'] : null;
        
         // Verifica o parâmetro "Inativar Consumidores na Importação"
        $inativaCons = $this->getInativaCons($CDLAYOUTIMP, $NRSEQTABELAY);
    
        // Verifica importação de mercado de boca
        $this->boImpMercadoBoca = $this->checkImpMercadoBoca($CDFILIAL);
    
        // Apaga consumidor auxiliar
        $this->deleteConsumidorAux();
        
        try {
            $this->entityManager->beginTransaction();
            
            // Itera sobre os consumidores e realiza as operações necessárias
            foreach ($consumers as $consumer) {
                $cdCliente = $consumer['CDCLIENTE'];
                $cdConsumidor = $consumer['CDCONSUMIDOR'];
                
                if ($consumer['IDVALIDO'] == 'N') {
                    continue;
                }
    
                // Verifica dados
                $this->verificaDados($consumer, $CDLAYOUTIMP, $NRSEQTABELAY);
                // Verifica consumidor
                $dadosConsumidor = $this->getVerificaConsumidor($cdCliente, $cdConsumidor);
    
                if (isset($dadosConsumidor[0])) {
                    $dadosConsumidor = $dadosConsumidor[0];
                }
    
                // Insere consumidor na tabela temporaria
                $this->insertConsumidorTemp($consumer, $dadosConsumidor);
            }

            // procedure importa consumidor, insere de fato na tabela consumidor
            $this->stpImportaCons($consumers, $CDLAYOUTIMP, $CDFILIAL, $inativaCons, $AJUSTASALDOCHECK, $CDOPERADOR);
            
            // itera novamente para verificar extrato e raliment
            foreach ($consumers as $consumer) {
                // Insere extrato do consumidor se não houver registro para o consumidor
                $this->handleExtratoCons($consumer, $CDFILIAL, $NRORG);
    
                // Insere e deleta da Raliment e Prodraliment
                $this->handleRalimentProdraliment($consumer);
            }
            
            
            // removido por hora, atualizar o layout com apenas o nome do arquivo nao é o comportamento do delphi
            // o delphi salva o caminho inteiro, que não temos no retail
            // $this->updateLayout($FILENAME, $CDLAYOUTIMP);
            
            $this->entityManager->commit();
    
            return array(
                "error" => false,
                "message" => "Importação realizada com sucesso."
            );;
        }
        catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    protected function getInativaCons($cdLayoutImp, $nrSeqTabelaY) {
        $sqlInativaCons = "
            SELECT IDINACONSARQIMP
              FROM FUNLAYOUTIMP
             WHERE CDLAYOUTIMP = :P_CDLAYOUTIMP
               AND (NRSEQTABELAY = :P_NRSEQTABELAY OR :P_NRSEQTABELAY = 'T')

        "; // modifique a query aqui
        $paramsInativaCons = [
            'P_CDLAYOUTIMP' => $cdLayoutImp,
            'P_NRSEQTABELAY' => $nrSeqTabelaY
        ];
        return $this->entityManager->getConnection()->fetchOne($sqlInativaCons, $paramsInativaCons);
    }

    protected function checkImpMercadoBoca($cdFilial) {
        $sqlChecaImpMercadoBoca = "
            SELECT 1
              FROM PARAVEND
             WHERE CDFILIAL = :P_CDFILIAL
               AND IDIMPMERCBOCA = 'S'

        "; // modifique a query aqui
        $paramsChecaImpMercadoBoca = [
            'P_CDFILIAL' => $cdFilial
        ];

        return !empty($this->entityManager->getConnection()->fetchAll($sqlChecaImpMercadoBoca, $paramsChecaImpMercadoBoca));
    }

    protected function deleteConsumidorAux() {
        $sqlApagaConsumidorAux = "DELETE FROM TRABCONSUMIDOR";
        $this->entityManager->getConnection()->executeQuery($sqlApagaConsumidorAux, []);
    }

    protected function verificaDados(&$consumer, $CDLAYOUTIMP, $NRSEQTABELAY) {
        // Verificar campos obrigatórios
        if (empty($consumer['CDCLIENTE'])) {
            throw new \Exception("O Código do Cliente não pode ser vazio.");
        }
    
        if (empty($consumer['CDCONSUMIDOR'])) {
            throw new \Exception("O código do consumidor não pode ser nulo.");
        }
    
        if (empty($consumer['NMCONSUMIDOR'])) {
            throw new \Exception("O nome do consumidor não pode ser vazio.");
        }
    
        if (empty($consumer['CDTIPOCONS'])) {
            throw new \Exception("O tipo de consumidor não pode ser vazio.");
        }
    
        if (empty($consumer['CDCCUSCLIE'])) {
            throw new \Exception("O centro de custo não pode ser vazio.");
        }
    
        if (!in_array($consumer['IDVERSALDCON'], ['S', 'N'])) {
            throw new \Exception("Identificador de verificação de saldo inválido. Se este identificador existir no layout deverá receber os valores 'S' ou 'N'.");
        }
    
        // Verificar identificador de consumidor
        if ($this->getInativaCons($CDLAYOUTIMP, $NRSEQTABELAY) === 'N') {
            $sqlVrIdentificador = "
                SELECT COUNT(*) AS NRREGISTROS
                FROM CONSUMIDOR
                WHERE CDIDCONSUMID = :P_CDIDCONSUMID
                AND CDIDCONSUMID IS NOT NULL
                AND IDSITCONSUMI = '1'
                AND CDCLIENTE = :P_CDCLIENTE
                AND CDCONSUMIDOR <> :P_CDCONSUMIDOR
            ";
            $paramsVrIdentificador = [
                'P_CDCONSUMIDOR' => $consumer['CDCONSUMIDOR'],
                'P_CDIDCONSUMID' => $this->getCdIdConsumidor($consumer['CDIDCONSUMID']),
                'P_CDCLIENTE' => $consumer['CDCLIENTE']
            ];
            $vrIdentificador = $this->entityManager->getConnection()->fetchAll($sqlVrIdentificador, $paramsVrIdentificador);
            if ($vrIdentificador[0]['NRREGISTROS'] > 0) {
                $cracha = $this->getCdIdConsumidor($consumer['CDIDCONSUMID']);
                throw new \Exception("Número do crachá ". $cracha . " já existe no cadastro, para outro consumidor.");
            }
        }
    
        // Verificar tipo de consumidor
        if (!empty($consumer['CDTIPOCONS'])) {
            $sqlTipoCons = "
                SELECT CDTIPOCONS
                FROM TIPOCONS
                WHERE CDTIPOCONS = :P_CDTIPOCONS
            ";
            $paramsTipoCons = ['P_CDTIPOCONS' => $consumer['CDTIPOCONS']];
            $tipoCons = $this->entityManager->getConnection()->fetchAll($sqlTipoCons, $paramsTipoCons);
        
            if (empty($tipoCons)) {
                $sqlInsertTipoCons = "
                    INSERT INTO TIPOCONS (CDTIPOCONS, NMTIPOCONS)
                    VALUES (:P_CDTIPOCONS, :P_NMTIPOCONS)
                ";
                $paramsInsertTipoCons = [
                    'P_CDTIPOCONS' => $consumer['CDTIPOCONS'],
                    'P_NMTIPOCONS' => $this->ajusta(date('Y-m-d H:i:s'), 20) // Ajusta DateTime
                ];
                $this->entityManager->getConnection()->executeQuery($sqlInsertTipoCons, $paramsInsertTipoCons);
            }
        }
    
        // Verificar centro de custo
        if (!empty($consumer['CDCCUSCLIE'])) {
            $sqlCcustoClie = "
                SELECT CDCLIENTE, CDCCUSCLIE
                FROM CCUSCLIE
                WHERE CDCLIENTE = :P_CDCLIENTE
                  AND CDCCUSCLIE = :P_CDCCUSCLIE
            ";
            $paramsCcustoClie = [
                'P_CDCLIENTE' => $consumer['CDCLIENTE'],
                'P_CDCCUSCLIE' => $consumer['CDCCUSCLIE']
            ];
            $ccustoClie = $this->entityManager->getConnection()->fetchAll($sqlCcustoClie, $paramsCcustoClie);
    
            if (empty($ccustoClie)) {
                $nmCcustoClie = !empty($consumer['NMCCUSCLIE']) ? $consumer['NMCCUSCLIE'] : 'N/A';
                $sqlInsertCcustoClie = "
                    INSERT INTO CCUSCLIE (CDCLIENTE, CDCCUSCLIE, NMCCUSCLIE)
                    VALUES (:P_CDCLIENTE, :P_CDCCUSCLIE, :P_NMCCUSCLIE)
                ";
                $paramsInsertCcustoClie = [
                    'P_CDCLIENTE' => $consumer['CDCLIENTE'],
                    'P_CDCCUSCLIE' => $consumer['CDCCUSCLIE'],
                    'P_NMCCUSCLIE' => $this->ajusta($nmCcustoClie, 50) // Ajusta o nome do centro de custo conforme necessário
                ];
                $this->entityManager->getConnection()->executeQuery($sqlInsertCcustoClie, $paramsInsertCcustoClie);
            }
        }
    
        // Verificar vendedor
        if (!empty($consumer['CDVENDEDOR'])) {
            $sqlVendedor = "
                SELECT CDVENDEDOR
                FROM VENDEDOR
                WHERE CDVENDEDOR = :P_CDVENDEDOR
            ";
            $paramsVendedor = ['P_CDVENDEDOR' => $consumer['CDVENDEDOR']];
            $vendedor = $this->entityManager->getConnection()->fetchAll($sqlVendedor, $paramsVendedor);
    
            if (empty($vendedor)) {
                $consumer['CDVENDEDOR'] = '';
            }
        }
    
        // Verificar país
        if (!empty($consumer['CDPAIS'])) {
            $sqlPais = "
                SELECT CDPAIS
                FROM PAIS
                WHERE CDPAIS = :P_CDPAIS
            ";
            $paramsPais = ['P_CDPAIS' => $consumer['CDPAIS']];
            $pais = $this->entityManager->getConnection()->fetchAll($sqlPais, $paramsPais);
            if (empty($pais)) {
                $consumer['CDPAIS'] = '';
            }
        }
        
        // Verificar estado
        if (!empty($consumer['SGESTADO'])) {
            $sqlEstado = "
                SELECT SGESTADO
                FROM ESTADO
                WHERE CDPAIS = :P_CDPAIS
                AND SGESTADO = :P_SGESTADO
            ";
            $paramsEstado = [
                'P_CDPAIS' => $consumer['CDPAIS'],
                'P_SGESTADO' => $consumer['SGESTADO']
            ];
            $estado = $this->entityManager->getConnection()->fetchAll($sqlEstado, $paramsEstado);
            if (empty($estado)) {
                $consumer['SGESTADO'] = '';
            }
        }
    
        // Verificar município
        if (!empty($consumer['CDMUNICIPIO'])) {
            $sqlMunicipio = "
                SELECT CDMUNICIPIO
                FROM MUNICIPIO
                WHERE CDPAIS = :P_CDPAIS
                AND SGESTADO = :P_SGESTADO
                AND CDMUNICIPIO = :P_CDMUNICIPIO
            ";
            $paramsMunicipio = [
                'P_CDPAIS' => $consumer['CDPAIS'],
                'P_SGESTADO' => $consumer['SGESTADO'],
                'P_CDMUNICIPIO' => $consumer['CDMUNICIPIO']
            ];
            $municipio = $this->entityManager->getConnection()->fetchAll($sqlMunicipio, $paramsMunicipio);
    
            if (empty($municipio)) {
                $consumer['CDMUNICIPIO'] = '';
            }
        }
    
        // Verificar bairro
        if (!empty($consumer['CDBAIRRO'])) {
            $sqlBairro = "
                SELECT CDBAIRRO
                FROM BAIRRO
                WHERE CDPAIS = :P_CDPAIS
                AND SGESTADO = :P_SGESTADO
                AND CDMUNICIPIO = :P_CDMUNICIPIO
                AND CDBAIRRO = :P_CDBAIRRO
            ";
            $paramsBairro = [
                'P_CDPAIS' => $consumer['CDPAIS'],
                'P_SGESTADO' => $consumer['SGESTADO'],
                'P_CDMUNICIPIO' => $consumer['CDMUNICIPIO'],
                'P_CDBAIRRO' => $consumer['CDBAIRRO']
            ];
            $bairro = $this->entityManager->getConnection()->fetchAll($sqlBairro, $paramsBairro);
    
            if (empty($bairro)) {
                $consumer['CDBAIRRO'] = '';
            }
        }
    }
    
    private function ajusta($stString, $inTamStr) {
	    $stString = trim($stString);
	    if (strlen($stString) != $inTamStr) {
	        if (!empty($stString)) {
	            for ($i = 0; $i < strlen($stString); $i++) {
	                if (!is_numeric($stString[$i]) && $stString[$i] !== '.') {
	                    return $stString;
	                }
	            }
	            while (strlen($stString) < $inTamStr) {
	                $stString = '0' . $stString;
	            }
	        }
	    }
	    return $stString;
	}
    
    protected function getCdIdConsumidor($P_CDIDCONSUMIDOR) {
        if ($this->boImpMercadoBoca) {
            $stTemp = str_replace(':', '', $P_CDIDCONSUMIDOR);
            $stResult = '';
            $stResult .= substr($stTemp, 6, 2);
            $stResult .= substr($stTemp, 4, 2);
            $stResult .= substr($stTemp, 2, 2);
            $stResult .= substr($stTemp, 0, 2);
            return $stResult;
        } else {
            return $P_CDIDCONSUMIDOR;
        }
    }


    protected function getVerificaConsumidor($cdCliente, $cdConsumidor) {
        $sqlVerificaConsumidor = "
            SELECT DSCOMPLENDECONS, NRENDECONS, NRTELE2CONS, NRCELULARCONS,
                   IDIMPCPFCUPOM, IDPERCONSPRODEX, IDCADCONFLIBCON, IDTPSELMANHA,
                   IDTPSELTARDE, IDTPSEALMOCO
              FROM CONSUMIDOR
             WHERE CDCLIENTE = :P_CDCLIENTE
               AND CDCONSUMIDOR = :P_CDCONSUMIDOR
        ";
        $paramsVerificaConsumidor = [
            'P_CDCLIENTE' => $cdCliente,
            'P_CDCONSUMIDOR' => $cdConsumidor
        ];
        return $this->entityManager->getConnection()->fetchAll($sqlVerificaConsumidor, $paramsVerificaConsumidor);
    }

    protected function insertConsumidorTemp($consumer, $verificaConsumidor) {
        $sqlInsConsumidor = "
            INSERT INTO TRABCONSUMIDOR (
                CDCLIENTETRAB, CDCONSUMIDORTRAB, NMCONSUMIDORTRAB, IDSITCONSUMITRAB,
                CDIDCONSUMIDTRAB, CDEXCONSUMIDTRAB, VRAVIDEBCONSTRAB, VRLIMDEBCONSTRAB,
                CDCCUSCLIETRAB, CDTIPOCONSTRAB, CDPAISTRAB, SGESTADOTRAB,
                CDMUNICIPIOTRAB, DSENDECONSTRAB, NMBAIRCONSTRAB, NRCEPCONSTRAB,
                NRTELE2CONSTRAB, NRTELECONSTRAB, NRRAMAL1CONSTRAB, NRRAMAL2CONSTRAB,
                NRFAXCONSTRAB, DSEMAILCONSTRAB, IDVERSALDCONTRAB, VRPESUBCONSTRAB,
                VRMESUBCONSTRAB, IDCONSUMIDORTRAB, IDATUCONSUMITRAB, VRDIASUBCONSTRAB,
                NMLOGINCONSTRAB, CDSENHASITETRAB, CDVENDEDORTRAB, NRRGCONSUMIDTRAB,
                CDSENHACONSTRAB, NRCPFRESPCONTRAB, NMRESPCONSTRAB, DTNASCCONSTRAB,
                VRMAXDEBCONSTRAB, VRLIMCREDCONSTRAB, VRAVICREDCONSTRAB, VRMAXCREDCONSTRAB,
                NRACESSOSITETRAB, IDTPVENDACONSTRAB, NRCELULARCONSTRAB, NRENDECONSTRAB,
                DSCOMPLENDECONSTRAB, IDIMPCPFCUPOMTRAB, IDPERCONSPRODEXTRAB, IDCADCONFLIBCONTRAB,
                DSREFERENDECONSTRAB, IDTPSELMANHATRAB, IDTPSEALMOCOTRAB, IDTPSELTARDETRAB,
                VRSUBSBASICOTRAB, IDBLOQSALDFSTRAB, CDFAMILISALDIMPTRAB, VRSALDOCREDIMPTRAB,
                VRMAXCONSMENSALTRAB, CDBAIRROTRAB
            ) VALUES (
                :CDCLIENTE, :CDCONSUMIDOR, :NMCONSUMIDOR, :IDSITCONSUMI,
                :CDIDCONSUMID, :CDEXCONSUMID, :VRAVIDEBCONS, :VRLIMDEBCONS,
                :CDCCUSCLIE, :CDTIPOCONS, :CDPAIS, :SGESTADO,
                :CDMUNICIPIO, :DSENDECONS, :NMBAIRCONS, :NRCEPCONS,
                :NRTELE2CONS, :NRTELECONS, :NRRAMAL1CONS, :NRRAMAL2CONS,
                :NRFAXCONS, :DSEMAILCONS, :IDVERSALDCON, :VRPESUBCONS,
                :VRMESUBCONS, :IDCONSUMIDOR, :IDATUCONSUMI, :VRDIASUBCONS,
                :NMLOGINCONS, :CDSENHASITE, :CDVENDEDOR, :NRRGCONSUMID,
                :CDSENHACONS, :NRCPFRESPCON, :NMRESPCONS, :DTNASCCONS,
                :VRMAXDEBCONS, :VRLIMCREDCONS, :VRAVICREDCONS, :VRMAXCREDCONS,
                :NRACESSOSITE, :IDTPVENDACONS, :NRCELULARCONS, :NRENDECONS,
                :DSCOMPLENDECONS, :IDIMPCPFCUPOM, :IDPERCONSPRODEX, :IDCADCONFLIBCON,
                :DSREFERENDECONS, :IDTPSELMANHA, :IDTPSEALMOCO, :IDTPSELTARDE,
                :VRSUBSBASICO, :IDBLOQSALDFS, :CDFAMILISALDIMP, :VRSALDOCREDIMP,
                :VRMAXCONSMENSAL, :CDBAIRRO
            )
        ";
        
        // validar CPF
        if (!empty($consumer['NRCPFRESPCON'])) {
            if (!$this->isCpfValid($consumer['NRCPFRESPCON']) || strlen($consumer['NRCPFRESPCON']) !== 11) {
                $consumer['NRCPFRESPCON'] = '';
            } else {
                $consumer['NRCPFRESPCON'] = substr($consumer['NRCPFRESPCON'], 0, 11);
            }
        } else {
            $consumer['NRCPFRESPCON'] = '';
        }
        
        $paramsInsConsumidor = [
            'CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
            'CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
            'NMCONSUMIDOR' => substr($consumer['NMCONSUMIDOR'], 0, 50),
            'IDSITCONSUMI' => '1',
            'CDIDCONSUMID' => $this->getCdIdConsumidor(substr($consumer['CDIDCONSUMID'], 0, 21)),
            'CDEXCONSUMID' => substr($consumer['CDEXCONSUMID'], 0, 21),
            'VRAVIDEBCONS' => $consumer['VRAVIDEBCONS'],
            'VRLIMDEBCONS' => $consumer['VRLIMDEBCONS'],
            'VRMAXDEBCONS' => $consumer['VRMAXDEBCONS'],
            'VRSUBSBASICO' => $consumer['VRSUBSBASICO'],
            'VRMAXCONSMENSAL' => $consumer['VRMAXCONSMENSAL'],
            'VRLIMCREDCONS' => $consumer['VRLIMCREDCONS'],
            'VRAVICREDCONS' => $consumer['VRAVICREDCONS'],
            'VRMAXCREDCONS' => $consumer['VRMAXCREDCONS'],
            'CDCCUSCLIE' => substr($consumer['CDCCUSCLIE'], 0, 10),
            'CDTIPOCONS' => substr($consumer['CDTIPOCONS'], 0, 2),
            'CDPAIS' => substr($consumer['CDPAIS'], 0, 4),
            'SGESTADO' => substr($consumer['SGESTADO'], 0, 2),
            'CDMUNICIPIO' => substr($consumer['CDMUNICIPIO'], 0, 4),
            'CDBAIRRO' => substr($consumer['CDBAIRRO'], 0, 10),
            'DSENDECONS' => substr($consumer['DSENDECONS'], 0, 50),
            'NMBAIRCONS' => substr($consumer['NMBAIRCONS'], 0, 30),
            'NRCEPCONS' => substr($consumer['NRCEPCONS'], 0, 15),
            'NRTELE2CONS' => empty($verificaConsumidor['NRTELE2CONS']) ? substr($consumer['NRTELE2CONS'], 0, 25) : $verificaConsumidor['NRTELE2CONS'],
            'NRTELECONS' => substr($consumer['NRTELECONS'], 0, 25),
            'NRRAMAL1CONS' => substr($consumer['NRRAMAL1CONS'], 0, 4),
            'NRRAMAL2CONS' => substr($consumer['NRRAMAL2CONS'], 0, 4),
            'NRFAXCONS' => substr($consumer['NRFAXCONS'], 0, 25),
            'DSEMAILCONS' => substr($consumer['DSEMAILCONS'], 0, 50),
            'VRPESUBCONS' => isset($consumer['VRPESUBCONS']) && $consumer['VRPESUBCONS'] !== '' ? $consumer['VRPESUBCONS'] : 0,
            'VRMESUBCONS' => isset($consumer['VRMESUBCONS']) && $consumer['VRMESUBCONS'] !== '' ? $consumer['VRMESUBCONS'] : 0,
            'IDVERSALDCON' => substr($consumer['IDVERSALDCON'], 0, 1),
            'VRDIASUBCONS' => isset($consumer['VRDIASUBCONS']) && $consumer['VRDIASUBCONS'] !== '' ? $consumer['VRDIASUBCONS'] : 0,
            'IDATUCONSUMI' => 'S',
            'IDCONSUMIDOR' => 'C',
            'CDVENDEDOR' => substr($consumer['CDVENDEDOR'], 0, 4),
            'NRRGCONSUMID' => substr($consumer['NRRGCONSUMID'], 0, 20),
            'CDSENHACONS' => isset($consumer['CDSENHACONS']) && $consumer['CDSENHACONS'] != "" ? $this->getDelphiPasswordEncrypted(substr($consumer['CDSENHACONS'], 0, 40)) : null,
            'NRCPFRESPCON' => $consumer['NRCPFRESPCON'],
            'NMRESPCONS' => substr($consumer['NMRESPCONS'], 0, 50),
            'NMLOGINCONS' => substr($consumer['NMLOGINCONS'], 0, 200),
            'DTNASCCONS' => $consumer['DTNASCCONS'],
            'CDSENHASITE' => isset($consumer['CDSENHASITE']) && $consumer['CDSENHASITE'] != "" ? $this->getDelphiPasswordEncrypted(substr($consumer['CDSENHASITE'], 0, 40)) : null,
            'NRACESSOSITE' => isset($consumer['NRACESSOSITE']) && $consumer['NRACESSOSITE'] !== '' ? $consumer['NRACESSOSITE'] : 0,
            'IDTPVENDACONS' => substr($consumer['IDTPVENDACONS'], 0, 1),
            'DSREFERENDECONS' => substr($consumer['DSREFERENDECONS'], 0, 2000),
            'NRENDECONS' => empty($verificaConsumidor['NRENDECONS']) ? substr($consumer['NRENDECONS'], 0, 5) : $verificaConsumidor['NRENDECONS'],
            'NRCELULARCONS' => empty($verificaConsumidor['NRCELULARCONS']) ? substr($consumer['NRCELULARCONS'], 0, 25) : $verificaConsumidor['NRCELULARCONS'],
            'DSCOMPLENDECONS' => empty($verificaConsumidor['DSCOMPLENDECONS']) ? substr($consumer['DSCOMPLENDECONS'], 0, 100) : $verificaConsumidor['DSCOMPLENDECONS'],
            'IDIMPCPFCUPOM' => empty($verificaConsumidor['IDIMPCPFCUPOM']) ? (empty($consumer['IDIMPCPFCUPOM']) ? 'S' : substr($consumer['IDIMPCPFCUPOM'], 0, 1)) : $verificaConsumidor['IDIMPCPFCUPOM'],
            'IDPERCONSPRODEX' => empty($verificaConsumidor['IDPERCONSPRODEX']) ? (empty($consumer['IDPERCONSPRODEX']) ? 'S' : substr($consumer['IDPERCONSPRODEX'], 0, 1)) : $verificaConsumidor['IDPERCONSPRODEX'],
            'IDCADCONFLIBCON' => empty($verificaConsumidor['IDCADCONFLIBCON']) ? (empty($consumer['IDCADCONFLIBCON']) ? 'N' : substr($consumer['IDCADCONFLIBCON'], 0, 1)) : $verificaConsumidor['IDCADCONFLIBCON'],
            'IDTPSELMANHA' => substr($consumer['IDTPSELMANHA'], 0, 1),
            'IDTPSELTARDE' => substr($consumer['IDTPSELTARDE'], 0, 1),
            'IDTPSEALMOCO' => substr($consumer['IDTPSEALMOCO'], 0, 1),
            'IDBLOQSALDFS' => substr($consumer['IDBLOQSALDFS'], 0, 1),
            'CDFAMILISALDIMP' => $consumer['CDFAMILISALDIMP'],
            'VRSALDOCREDIMP' => $consumer['VRSALDOCREDIMP'],
            'VRMAXCONSMENSAL' => $consumer['VRMAXCONSMENSAL']
        ];
    
        $this->entityManager->getConnection()->executeQuery($sqlInsConsumidor, $paramsInsConsumidor);
    }
    
    private function stpImportaCons($consumers, $cdLayoutImp, $CDFILIAL, $inativaCons, $AJUSTASALDOCHECK, $CDOPERADOR) {
        $layoutFields = [
            'CDIDCONSUMID'   => 'P_boCdIdConsumid', 'CDEXCONSUMID'   => 'P_boCdExConsumid', 'NMCONSUMIDOR'   => 'P_boNmConsumidor', 'VRAVIDEBCONS'   => 'P_boVrAviDebCons',
            'VRLIMDEBCONS'   => 'P_boVrLimDebCons', 'DSENDECONS'     => 'P_boDsEndeCons', 'NMBAIRCONS'     => 'P_boNmBairCons', 'NRCEPCONS'      => 'P_boNrCepCons',
            'NRTELE2CONS'    => 'P_boNrTele2Cons', 'NRTELECONS'     => 'P_boNrTeleCons', 'NRRAMAL1CONS'   => 'P_boNrRamal1Cons', 'NRRAMAL2CONS'   => 'P_boNrRamal2Cons',
            'NRFAXCONS'      => 'P_boNrFaxCons', 'DSEMAILCONS'    => 'P_boDsEmailCons', 'VRPESUBCONS'    => 'P_boVrPeSubCons', 'VRMESUBCONS'    => 'P_boVrMeSubCons',
            'CDTIPOCONS'     => 'P_boCdTipoCons', 'CDCCUSCLIE'     => 'P_boCdCCusClie', 'IDVERSALDCON'   => 'P_boIdVerSaldCon', 'VRDIASUBCONS'   => 'P_boVrDiaSubCons',
            'CDVENDEDOR'     => 'P_boCdVendedor', 'NRRGCONSUMID'   => 'P_boNrRgConsumid', 'CDSENHACONS'    => 'P_boCdSenhaCons', 'NRCPFRESPCON'   => 'P_boNrCpfRespCon',
            'NMRESPCONS'     => 'P_boNmRespCons', 'NMLOGINCONS'    => 'P_boNmLoginCons', 'DTNASCCONS'     => 'P_boDtNasCCons', 'VRMAXDEBCONS'   => 'P_boVrMaxDebCons',
            'VRLIMCREDCONS'  => 'P_boVrLimCredCons', 'VRAVICREDCONS'  => 'P_boVrAviCredCons', 'VRMAXCREDCONS'  => 'P_boVrMaxCredCons', 'CDSENHASITE'    => 'P_boCdSenhaSite',
            'NRACESSOSITE'   => 'P_boNrAcessoSite', 'CDPAIS'         => 'P_boCdPais', 'SGESTADO'       => 'P_boSgEstado', 'CDMUNICIPIO'    => 'P_boCdMunicipio',
            'IDTPVENDACONS'  => 'P_boIdTpVendaCons', 'NRENDECONS'     => 'P_boNrEndeCons', 'NRCELULARCONS'  => 'P_boNrCelularCons', 'DSCOMPLENDECONS'=> 'P_boDsComplEndeCons',
            'DSREFERENDECONS'=> 'P_boDsReferEndeCons', 'IDIMPCPFCUPOM'  => 'P_boIdImpCpfCupom', 'IDPERCONSPRODEX'=> 'P_boIdPerConsProdEx', 'IDCADCONFLIBCON'=> 'P_boIdCadConfLibCon',
            'IDTPSELMANHA'   => 'P_boIdTpSeLManha', 'IDTPSELTARDE'   => 'P_boIdTpSeLTarde', 'IDTPSEALMOCO'   => 'P_boIdTpSeAlmoco', 'VRSUBSBASICO'   => 'P_boVrSubsBasico',
            'VRMAXCONSMENSAL'=> 'P_boVrMaxConsMensal','IDBLOQSALDFS'   => 'P_boIdBloqSaldFs', 'CDFAMILISALDIMP'=> 'P_boCdFamiliSaldImp', 'VRSALDOCREDIMP' => 'P_boVrSaldoCredImp',
            'CDBAIRRO'       => 'P_boCdBairro'
        ];
    
        $paramsOracle = [];
    
        foreach ($layoutFields as $field => $varName) {
            $paramsOracle[$varName] = $this->layout($cdLayoutImp, $field) ? 'S' : '';
        }
    
        $paramsOracle['P_CDFILIAL'] = $CDFILIAL;
        $paramsOracle['P_CDCLIENTE'] = $this->getCdCliente($consumers);
        $paramsOracle['P_INATIVACONS'] = $inativaCons;
        
        // verifica se o cliente e a filial estao em CLIENFILIAL para definir se é para inserir em CONSUMIFILIAL
        // sem erro de restricao de PK
        if ($this->verificaClienteFilial($paramsOracle['P_CDCLIENTE'], $paramsOracle['P_CDFILIAL'])) {
            $paramsOracle['P_MODOFILIAL'] = 'S';
        } else {
            $paramsOracle['P_MODOFILIAL'] = 'N';
        }
        
        $paramsOracle['P_IDAJUSALDIMPCON'] = $this->findAjustaSaldo($CDFILIAL);
        $paramsOracle['P_AJUSALDCONSNOVO'] = $AJUSTASALDOCHECK ? 'S' : 'N';
        $paramsOracle['P_CDOPERADOR'] = $CDOPERADOR;
    
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'IMPORTA_CONSUMIDOR');
    
        // ordenar parametros para a procedure
        $orderedParams = [
            'P_boCdIdConsumid', 'P_boCdExConsumid', 'P_boNmConsumidor', 'P_boVrAviDebCons', 'P_boVrLimDebCons', 
            'P_boDsEndeCons', 'P_boNmBairCons', 'P_boNrCepCons', 'P_boNrTele2Cons', 'P_boNrTeleCons', 
            'P_boNrRamal1Cons', 'P_boNrRamal2Cons', 'P_boNrFaxCons', 'P_boDsEmailCons', 'P_boVrPeSubCons', 
            'P_boVrMeSubCons', 'P_boCdTipoCons', 'P_boCdCCusClie', 'P_boIdVerSaldCon', 'P_boVrDiaSubCons', 
            'P_boCdVendedor', 'P_boNrRgConsumid', 'P_boCdSenhaCons', 'P_boNrCpfRespCon', 'P_boNmRespCons', 
            'P_boNmLoginCons', 'P_boDtNasCCons', 'P_boVrMaxDebCons', 'P_boVrLimCredCons', 'P_boVrAviCredCons', 
            'P_boVrMaxCredCons', 'P_boCdSenhaSite', 'P_boNrAcessoSite', 'P_boCdPais', 'P_boSgEstado', 
            'P_boCdMunicipio', 'P_boIdTpVendaCons', 'P_boNrEndeCons', 'P_boNrCelularCons', 'P_boDsComplEndeCons', 
            'P_boIdImpCpfCupom', 'P_boIdPerConsProdEx', 'P_boIdCadConfLibCon', 'P_boDsReferEndeCons', 
            'P_CDFILIAL', 'P_CDCLIENTE', 'P_INATIVACONS', 'P_MODOFILIAL', 'P_boIdTpSeLManha', 
            'P_boIdTpSeLTarde', 'P_boIdTpSeAlmoco', 'P_boVrSubsBasico', 'P_boIdBloqSaldFs', 
            'P_boCdFamiliSaldImp', 'P_boVrSaldoCredImp', 'P_boVrMaxConsMensal', 'P_boCdBairro', 
            'P_IDAJUSALDIMPCON', 'P_AJUSALDCONSNOVO', 'P_CDOPERADOR'
        ];
    
        foreach ($orderedParams as $paramName) {
            $procedure->addParam(new Param($paramName, Param::PARAM_INPUT, $paramsOracle[$paramName], Param::PARAM_TYPE_STR));
        }
    
        $procedure->execute();
    }

    private function verificaClienteFilial($cdCliente, $cdFilial) {
        // Query para verificar se o cliente e a filial existem na tabela CLIENFILIAL
        $sqlVerifica = "
            SELECT COUNT(*) AS COUNT
            FROM CLIENFILIAL
            WHERE CDCLIENTE = :P_CDCLIENTE AND CDFILIAL = :P_CDFILIAL
        ";
        
        // Parâmetros para a query
        $paramsVerifica = [
            'P_CDCLIENTE' => $cdCliente,
            'P_CDFILIAL' => $cdFilial
        ];
        
        // Executa a query
        $result = $this->entityManager->getConnection()->fetchAssoc($sqlVerifica, $paramsVerifica);
        
        // Retorna true se o cliente e a filial existirem, caso contrário, false
        return $result['COUNT'] > 0;
    }

        
    private function layout($cdLayoutImp, $nmColunaImp) {
        $sql = "SELECT COUNT(*) AS NRREGISTROS FROM COLLAYOUTIMP WHERE CDLAYOUTIMP = :P_CDLAYOUTIMP AND NMCOLUNAIMP = :P_NMCOLUNAIMP";
        $params = [
            'P_CDLAYOUTIMP' => $cdLayoutImp,
            'P_NMCOLUNAIMP' => $nmColunaImp
        ];
        $result = $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        return $result['NRREGISTROS'] > 0;
    }
    
   private function getCdCliente($consumers) {
        $cdClienteSet = [];
        foreach ($consumers as $consumer) {
            $cdClienteSet[] = (array) $consumer; // Convert object to array
        }
        $cdClienteSet = array_unique(array_column($cdClienteSet, 'CDCLIENTE'));
        if (count($cdClienteSet) === 1) {
            return $cdClienteSet[0];
        }
        return implode('_', $cdClienteSet);
    }


    protected function handleExtratoCons($consumer, $filial, $NRORG) {
        // Verifica se a família informada no arquivo existe
        if (!empty($consumer['CDFAMILISALDIMP'])) {
            $sqlVerificaFamiliaSaldo = "
                SELECT CDFAMILISALD
                FROM FAMILIASALDO
                WHERE CDFAMILISALD = :P_CDFAMILISALD 
            ";
            $paramsVerificaFamiliaSaldo = [
                'P_CDFAMILISALD' => substr($consumer['CDFAMILISALDIMP'], 0, 3)
            ];
            $verificaFamiliaSaldo = $this->entityManager->getConnection()->fetchAll($sqlVerificaFamiliaSaldo, $paramsVerificaFamiliaSaldo);
    
            if (!empty($verificaFamiliaSaldo)) {
                $sqlVerificaExtratocons = "
                    SELECT COUNT(*) AS REGISTROS
                    FROM EXTRATOCONS
                    WHERE CDCLIENTE = :P_CDCLIENTE
                      AND CDCONSUMIDOR = :P_CDCONSUMIDOR
                      AND CDFAMILISALD = :P_CDFAMILISALD
                ";
                $paramsVerificaExtratocons = [
                    'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                    'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                    'P_CDFAMILISALD' => substr($consumer['CDFAMILISALDIMP'], 0, 3)
                ];
                $verificaExtratocons = $this->entityManager->getConnection()->fetchAll($sqlVerificaExtratocons, $paramsVerificaExtratocons);
    
                if ($verificaExtratocons[0]['REGISTROS'] == 0) {
                    $sqlInsMovExtrato = "
                        INSERT INTO EXTRATOCONS (
                            CDCLIENTE, CDCONSUMIDOR, CDFAMILISALD, DTMOVEXTCONS, NRSEQMOVEXT, CDTIPORECE,
                            DSOPEEXTCONS, VRMOVEXTCONS, VRSALDCONEXT, IDTPMOVEXT, DTULTATUEXT, NRSEQREGCREDVND,
                            IDIMPEXTRATO, CDFILIAL
                        )
                        VALUES (
                            :CDCLIENTE, :CDCONSUMIDOR, :CDFAMILISALD, :DTMOVEXTCONS, :NRSEQMOVEXT, :CDTIPORECE,
                            :DSOPEEXTCONS, :VRMOVEXTCONS, :VRSALDCONEXT, :IDTPMOVEXT, SYSDATE, :NRSEQREGCREDVND,
                            'S', :CDFILIAL
                        )
                    ";
                    
                    $dtInsereSaldo = (new \DateTime())->format('d/m/Y');

                    $paramsInsMovExtrato = [
                        'CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                        'CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                        'CDFAMILISALD' => substr($consumer['CDFAMILISALDIMP'], 0, 3),
                        'DTMOVEXTCONS' => $dtInsereSaldo,
                        'NRSEQMOVEXT' => '001',
                        'CDTIPORECE' => $this->getTipoRece(), // Defina a função para obter o tipo de recebimento correto
                        'VRSALDCONEXT' => isset($consumer['VRSALDOCREDIMP']) && $consumer['VRSALDOCREDIMP'] !== '' ? $consumer['VRSALDOCREDIMP'] : 0,
                        'CDFILIAL' => $filial,
                        'DSOPEEXTCONS' => 'Crédito via Importação',
                        'IDTPMOVEXT' => 'C',
                        'VRMOVEXTCONS' => isset($consumer['VRSALDOCREDIMP']) && $consumer['VRSALDOCREDIMP'] !== '' ? $consumer['VRSALDOCREDIMP'] : 0,
                        'NRSEQREGCREDVND' => $this->novoCodigo->getCodeByTableName('EXTRATOCONSVND', $NRORG)
                    ];
                    $this->entityManager->getConnection()->executeQuery($sqlInsMovExtrato, $paramsInsMovExtrato);
    
    
                    $conditions = array(
            		    'CDCLIENTE'     => substr($consumer['CDCLIENTE'], 0, 21),
            		    'CDCONSUMIDOR'  => substr($consumer['CDCONSUMIDOR'], 0, 21),
            		    'DTMOVEXTCONS'  => $dtInsereSaldo,
            		    'NRORG'         => ''
            		); 
                    $this->evoluiSaldoService->EvoluiSaldoFunction($conditions);
                }
            }
        }
    }
    
    public function getTipoRece() {
        $sqlTipoRece = "
            SELECT A.CDTIPORECE
            FROM (
                SELECT MIN(CDTIPORECE) AS CDTIPORECE
                FROM TIPORECE
                WHERE IDTIPORECE = '4'
            ) A
        ";

        $result = $this->entityManager->getConnection()->fetchAll($sqlTipoRece);

        if (!empty($result)) {
            return $result[0]['CDTIPORECE'];
        }

        return null;
    }

    // Função para lidar com Raliment e Prodraliment
    protected function handleRalimentProdraliment($consumer) {
        $sqlGetExtra = "
            SELECT CDPRODUTO, CDFAMILISALD, FI.CDFILIAL
            FROM FAMSALDOPROD FA, CLIENFILIAL FI
            WHERE FI.CDCLIENTE = :P_CDCLIENTE
              AND FA.CDFILIAL = FI.CDFILIAL
              AND FA.IDPERMREFEXTRA = 'S'
        ";
        $paramsGetExtra = [
            'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21)
        ];
        $extraRecords = $this->entityManager->getConnection()->fetchAll($sqlGetExtra, $paramsGetExtra);

        $sqlVerificaConsumidor = "
            SELECT DSCOMPLENDECONS, NRENDECONS, NRTELE2CONS, NRCELULARCONS,
                   IDIMPCPFCUPOM, IDPERCONSPRODEX, IDCADCONFLIBCON, IDTPSELMANHA,
                   IDTPSELTARDE, IDTPSEALMOCO
            FROM CONSUMIDOR
            WHERE CDCLIENTE = :P_CDCLIENTE
              AND CDCONSUMIDOR = :P_CDCONSUMIDOR
        ";
        
        $paramsVerificaConsumidor = [
            'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
            'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21)
        ];
        $verificaConsumidor = $this->entityManager->getConnection()->fetchAll($sqlVerificaConsumidor, $paramsVerificaConsumidor);

        if ($verificaConsumidor[0]['IDPERCONSPRODEX'] == 'S') {
            // DELETA
            foreach ($extraRecords as $extraRecord) {
                $sqlDelExtra = "
                    DELETE FROM PRODRALIMENT
                    WHERE CDFILIAL = :P_CDFILIAL
                      AND CDCLIENTE = :P_CDCLIENTE
                      AND CDCONSUMIDOR = :P_CDCONSUMIDOR
                      AND CDFAMILISALD = :P_CDFAMILISALD
                      AND CDPRODUTO = :P_CDPRODUTO
                ";
                $paramsDelExtra = [
                    'P_CDFILIAL' => $extraRecord['CDFILIAL'],
                    'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                    'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                    'P_CDFAMILISALD' => $extraRecord['CDFAMILISALD'],
                    'P_CDPRODUTO' => $extraRecord['CDPRODUTO']
                ];
                $this->entityManager->getConnection()->executeQuery($sqlDelExtra, $paramsDelExtra);

                $sqlExisteProd = "
                    SELECT CDFILIAL, CDCLIENTE, CDCONSUMIDOR, CDFAMILISALD
                    FROM PRODRALIMENT
                    WHERE CDFILIAL = :P_CDFILIAL
                      AND CDCLIENTE = :P_CDCLIENTE
                      AND CDCONSUMIDOR = :P_CDCONSUMIDOR
                      AND CDFAMILISALD = :P_CDFAMILISALD
                ";
                $paramsExisteProd = [
                    'P_CDFILIAL' => $extraRecord['CDFILIAL'],
                    'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                    'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                    'P_CDFAMILISALD' => $extraRecord['CDFAMILISALD']
                ];
                $existeProd = $this->entityManager->getConnection()->fetchAll($sqlExisteProd, $paramsExisteProd);

                if (empty($existeProd)) {
                    $sqlDelRaliment = "
                        DELETE FROM RALIMENT
                        WHERE CDFILIAL = :P_CDFILIAL
                          AND CDCLIENTE = :P_CDCLIENTE
                          AND CDCONSUMIDOR = :P_CDCONSUMIDOR
                          AND CDFAMILISALD = :P_CDFAMILISALD
                    ";
                    $paramsDelRaliment = [
                        'P_CDFILIAL' => $extraRecord['CDFILIAL'],
                        'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                        'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                        'P_CDFAMILISALD' => $extraRecord['CDFAMILISALD']
                    ];
                    $this->entityManager->getConnection()->executeQuery($sqlDelRaliment, $paramsDelRaliment);
                }
            }
        } else if ($verificaConsumidor[0]['IDPERCONSPRODEX'] == 'N') {
            // INSERE
            foreach ($extraRecords as $extraRecord) {
                $sqlExtraExiste = "
                    SELECT CDFILIAL, CDCLIENTE, CDCONSUMIDOR, CDFAMILISALD
                    FROM RALIMENT
                    WHERE CDFILIAL = :P_CDFILIAL
                      AND CDCLIENTE = :P_CDCLIENTE
                      AND CDCONSUMIDOR = :P_CDCONSUMIDOR
                      AND CDFAMILISALD = :P_CDFAMILISALD
                ";
                $paramsExtraExiste = [
                    'P_CDFILIAL' => $extraRecord['CDFILIAL'],
                    'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                    'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                    'P_CDFAMILISALD' => $extraRecord['CDFAMILISALD']
                ];
                $extraExiste = $this->entityManager->getConnection()->fetchAll($sqlExtraExiste, $paramsExtraExiste);

                if (empty($extraExiste)) {
                    $sqlInsRaliment = "
                        INSERT INTO RALIMENT (
                            CDFILIAL, CDCLIENTE, CDCONSUMIDOR, CDFAMILISALD, IDSALDNEGRAL
                        ) VALUES (
                            :P_CDFILIAL, :P_CDCLIENTE, :P_CDCONSUMIDOR, :P_CDFAMILISALD, 'S'
                        )
                    ";
                    $paramsInsRaliment = [
                        'P_CDFILIAL' => $extraRecord['CDFILIAL'],
                        'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                        'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                        'P_CDFAMILISALD' => $extraRecord['CDFAMILISALD']
                    ];
                    $this->entityManager->getConnection()->executeQuery($sqlInsRaliment, $paramsInsRaliment);
                }

                $sqlDelExtra = "
                    DELETE FROM PRODRALIMENT
                    WHERE CDFILIAL = :P_CDFILIAL
                      AND CDCLIENTE = :P_CDCLIENTE
                      AND CDCONSUMIDOR = :P_CDCONSUMIDOR
                      AND CDFAMILISALD = :P_CDFAMILISALD
                      AND CDPRODUTO = :P_CDPRODUTO
                ";
                $paramsDelExtra = [
                    'P_CDFILIAL' => $extraRecord['CDFILIAL'],
                    'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                    'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                    'P_CDFAMILISALD' => $extraRecord['CDFAMILISALD'],
                    'P_CDPRODUTO' => $extraRecord['CDPRODUTO']
                ];
                $this->entityManager->getConnection()->executeQuery($sqlDelExtra, $paramsDelExtra);

                $sqlInsExtra = "
                    INSERT INTO PRODRALIMENT (
                        CDFILIAL, CDCLIENTE, CDCONSUMIDOR, CDFAMILISALD, CDPRODUTO, QTCONSDIARAL
                    ) VALUES (
                        :P_CDFILIAL, :P_CDCLIENTE, :P_CDCONSUMIDOR, :P_CDFAMILISALD, :P_CDPRODUTO, 1
                    )
                ";
                $paramsInsExtra = [
                    'P_CDFILIAL' => $extraRecord['CDFILIAL'],
                    'P_CDCLIENTE' => substr($consumer['CDCLIENTE'], 0, 21),
                    'P_CDCONSUMIDOR' => substr($consumer['CDCONSUMIDOR'], 0, 21),
                    'P_CDFAMILISALD' => $extraRecord['CDFAMILISALD'],
                    'P_CDPRODUTO' => $extraRecord['CDPRODUTO']
                ];
                $this->entityManager->getConnection()->executeQuery($sqlInsExtra, $paramsInsExtra);
            }
        }
    }

    protected function updateLayout($nmArquivo, $cdLayoutImp) {
        $sqlUpdLayout = "
            UPDATE TABLAYOUTIMP
            SET NMARQLAYIMP = :P_NMARQUIVO
            WHERE CDLAYOUTIMP = :P_CDLAYOUTIMP
        ";
        $paramsUpdLayout = [
            'P_NMARQUIVO' => $nmArquivo,
            'P_CDLAYOUTIMP' => $cdLayoutImp
        ];
        $this->entityManager->getConnection()->executeQuery($sqlUpdLayout, $paramsUpdLayout);
    }
    
    public function getDelphiPasswordEncrypted($password) {
        // Parameters to retrieve the crypted password from a server 
        $options = array(
            'encoding' => 'ISO-8859-1',
            'trace' => true,
            'exceptions' => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', $this->xmlSchemaUrl)
        );

        $params = array(
            'rdm' => 'Bib_html.rdmDBGeralHtm',
            'method' => 'GetCriptoPass',
            'args' => $args,
        );
        
        // Tries to get the encrypted password
        try {
            $client = new \SoapClient($this->criptoPasswordUrl, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
        
        return $encryption;
    }
}