<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RelCampanhaFidelidadeService {
    protected $entityManager;

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getDSCAMPANHA($CDCAMPANHA){
        $params = array (
            'P_CDCAMPANHA' => $CDCAMPANHA
        );
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_DSCAMPANHA, $params);
    }
    
    public function getVIGCAMP($CDCAMPANHA){
        $params = array(
            'P_CDCAMPANHA' => $CDCAMPANHA
        );
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_VIGCAMP, $params);
    }
    
    public function getNRCONSUMIDORES($CDCAMPANHA, $DATAHOJE){
        $params = array(
            'P_CDCAMPANHA' => $CDCAMPANHA,
            'P_DATAHOJE' => $DATAHOJE
        );
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_NRCONSUMIDORES, $params);
    }
    
    public function getVRTOTVENDAS($CDCAMPANHA, $DTINIVIGCAMP, $DTFIMVIGCAMP){
        $params = array(
            'P_CDCAMPANHA' => $CDCAMPANHA,
            'P_DTINIVIGCAMP' => $DTINIVIGCAMP,
            'P_DTFIMVIGCAMP' => $DTFIMVIGCAMP
        );
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_VRTOTVENDAS, $params);
    }
    
    public function getNRBENEFICIADOS($CDCAMPANHA, $NRCONSUMIDORES, $DTINIVIGCAMP, $DTFIMVIGCAMP){
        $params = array(
            'P_CDCAMPANHA' => $CDCAMPANHA,
            'P_NRCONSUMIDORES' => $NRCONSUMIDORES,
            'P_DTINIVIGCAMP' => $DTINIVIGCAMP,
            'P_DTFIMVIGCAMP' => $DTFIMVIGCAMP
        );
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_NRBENEFICIADOS, $params);
    }

    public function getNRBRINDERESG($CDCAMPANHA, $DTINIVIGCAMP, $DTFIMVIGCAMP){
        $params = array(
            'P_CDCAMPANHA' => $CDCAMPANHA,
            'P_DTINIVIGCAMP' => $DTINIVIGCAMP,
            'P_DTFIMVIGCAMP' => $DTFIMVIGCAMP
        );
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_NRBRINDERESG, $params);
    }


	const GET_DSCAMPANHA = "
	    SELECT
           CDCAMPANHA, DSCAMPANHA
        FROM
           CAMPANHA
        WHERE
            CDCAMPANHA = :P_CDCAMPANHA
	";
	
	const GET_VIGCAMP = "
	    SELECT
           DTINIVIGCAMP, DTFIMVIGCAMP
        FROM
           CAMPANHA
        WHERE
           CDCAMPANHA = :P_CDCAMPANHA
	";
	
	
	// PEGAR P_DATAHOJE
	const GET_NRCONSUMIDORES = "
        SELECT
           COUNT(DISTINCT CDCLIENTE||CDCONSUMIDOR) AS NRCONSUMIDORES
        FROM
           MOVPONTOCONS
        WHERE
           (CDCAMPANHA = :P_CDCAMPANHA) AND
           ((DTVALIDAPONTO >= :P_DATAHOJE) OR (DTVALIDAPONTO IS NULL))
	";
	
	// P_DTINIVIGCAMP e P_DTFIMVIGCAMP
	const GET_VRTOTVENDAS = "
        SELECT
           NVL(A.VRTOTVENDAS,0) AS VRTOTVENDAS
        FROM
           (SELECT
               SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) AS VRTOTVENDAS
            FROM
               VENDA V, ITEMVENDA I, MOVPONTOCONS M, PRODUTO P
            WHERE
               (V.CDFILIAL    = I.CDFILIAL   ) AND
               (V.CDCAIXA     = I.CDCAIXA    ) AND
               (V.NRSEQVENDA  = I.NRSEQVENDA ) AND
               (V.CDFILIAL    = M.CDFILIAL   ) AND
               (V.CDCAIXA     = M.CDCAIXA    ) AND
               (V.NRSEQVENDA  = M.NRSEQVENDA ) AND
               (V.IDSITUVENDA = 'O'          ) AND
               (M.CDCAMPANHA  = :P_CDCAMPANHA) AND
               (M.IDTPMOVCONS = 'C'          ) AND
               (P.CDPRODUTO   = I.CDPRODUTO  ) AND
               (P.CDARVPROD LIKE (SELECT MAX(O.CDARVPROD) AS CDARVPROD
                                    FROM PRODCAMPANHA C, PRODUTO O
                                   WHERE C.CDPRODUTO  = O.CDPRODUTO
                                     AND C.CDCAMPANHA = :P_CDCAMPANHA
                                     AND INSTR(P.CDARVPROD, O.CDARVPROD) = 1)||'%') AND
               (V.DTENTRVENDA BETWEEN :P_DTINIVIGCAMP AND :P_DTFIMVIGCAMP)) A
	";
	
	// P_NRCONSUMIDORES, P_DTINIVIGCAMP e P_DTFIMVIGCAMP
	const GET_NRBENEFICIADOS = "
        SELECT
           COUNT(DISTINCT M.CDCLIENTE||M.CDCONSUMIDOR) AS NRBENEFICIADOS,
           (:P_NRCONSUMIDORES - COUNT(DISTINCT M.CDCLIENTE||M.CDCONSUMIDOR)) AS NRNAOBENEFICIADOS
        FROM
           MOVPONTOCONS M, VENDA V
        WHERE
           (M.CDCAMPANHA  = :P_CDCAMPANHA) AND
           (M.CDFILIAL    = V.CDFILIAL   ) AND
           (M.CDCAIXA     = V.CDCAIXA    ) AND
           (M.NRSEQVENDA  = V.NRSEQVENDA ) AND
           (M.IDTPMOVCONS = 'D'          ) AND
           (V.IDSITUVENDA = 'O'          ) AND
           (V.DTENTRVENDA BETWEEN :P_DTINIVIGCAMP AND :P_DTFIMVIGCAMP)
	";
	
	
	const GET_NRBRINDERESG = "
    	SELECT
           COUNT(*) AS NRBRINDERESG,
           NVL(SUM(TRUNC(I.VRUNITVEND * I.QTPRODVEND,2)),0) AS VRTOTBRINDERESG
        FROM
           VENDA V, ITEMVENDA I, MOVPONTOCONS M
        WHERE
           (V.CDFILIAL    = I.CDFILIAL   ) AND
           (V.CDCAIXA     = I.CDCAIXA    ) AND
           (V.NRSEQVENDA  = I.NRSEQVENDA ) AND
           (V.CDFILIAL    = M.CDFILIAL   ) AND
           (V.CDCAIXA     = M.CDCAIXA    ) AND
           (V.NRSEQVENDA  = M.NRSEQVENDA ) AND
           (V.IDSITUVENDA = 'O'          ) AND
           (M.CDCAMPANHA  = :P_CDCAMPANHA) AND
           (M.IDTPMOVCONS = 'D'          ) AND
           (V.DTENTRVENDA BETWEEN :P_DTINIVIGCAMP AND :P_DTFIMVIGCAMP)
	";
}
